@@ -457,7 +457,8 @@ from calibre.devices.blackberry.driver import BLACKBERRY
 from calibre.devices.cybook.driver import CYBOOK, ORIZON
 from calibre.devices.eb600.driver import EB600, COOL_ER, SHINEBOOK, \
                 POCKETBOOK360, GER2, ITALICA, ECLICTO, DBOOK, INVESBOOK, \
-                BOOQ, ELONEX, POCKETBOOK301, MENTOR, POCKETBOOK602
+                BOOQ, ELONEX, POCKETBOOK301, MENTOR, POCKETBOOK602, \
+                POCKETBOOK701
 from calibre.devices.iliad.driver import ILIAD
 from calibre.devices.irexdr.driver import IREXDR1000, IREXDR800
 from calibre.devices.jetbook.driver import JETBOOK, MIBUK, JETBOOK_MINI
@@ -473,12 +474,14 @@ from calibre.devices.binatone.driver import README
 from calibre.devices.hanvon.driver import N516, EB511, ALEX, AZBOOKA, THEBOOK
 from calibre.devices.edge.driver import EDGE
 from calibre.devices.teclast.driver import TECLAST_K3, NEWSMY, IPAPYRUS, \
-        SOVOS, PICO
+        SOVOS, PICO, SUNSTECH_EB700
 from calibre.devices.sne.driver import SNE
 from calibre.devices.misc import PALMPRE, AVANT, SWEEX, PDNOVEL, KOGAN, \
-        GEMEI, VELOCITYMICRO, PDNOVEL_KOBO, Q600, LUMIREAD
+        GEMEI, VELOCITYMICRO, PDNOVEL_KOBO, Q600, LUMIREAD, ALURATEK_COLOR, \
+        TREKSTOR, EEEREADER, NEXTBOOK
 from calibre.devices.folder_device.driver import FOLDER_DEVICE_FOR_CONFIG
 from calibre.devices.kobo.driver import KOBO
+from calibre.devices.bambook.driver import BAMBOOK
 
 from calibre.ebooks.metadata.fetch import GoogleBooks, ISBNDB, Amazon, \
     LibraryThing
@@ -545,9 +548,7 @@ plugins += [
     JETBOOK_MINI,
     MIBUK,
     SHINEBOOK,
-    POCKETBOOK360,
-    POCKETBOOK301,
-    POCKETBOOK602,
+    POCKETBOOK360, POCKETBOOK301, POCKETBOOK602, POCKETBOOK701,
     KINDLE,
     KINDLE2,
     KINDLE_DX,
@@ -580,7 +581,7 @@ plugins += [
     ELONEX,
     TECLAST_K3,
     NEWSMY,
-    PICO,
+    PICO, SUNSTECH_EB700,
     IPAPYRUS,
     SOVOS,
     EDGE,
@@ -601,6 +602,11 @@ plugins += [
     VELOCITYMICRO,
     PDNOVEL_KOBO,
     LUMIREAD,
+    ALURATEK_COLOR,
+    BAMBOOK,
+    TREKSTOR,
+    EEEREADER,
+    NEXTBOOK,
     ITUNES,
 ]
 plugins += [x for x in list(locals().values()) if isinstance(x, type) and \
