@@ -18,9 +18,9 @@ class FOLDER_DEVICE_FOR_CONFIG(USBMS):
     supported_platforms = ['windows', 'osx', 'linux']
     FORMATS     = ['epub', 'fb2', 'mobi', 'azw', 'lrf', 'tcr', 'pmlz', 'lit',
                    'rtf', 'rb', 'pdf', 'oeb', 'txt', 'pdb', 'prc']
-    VENDOR_ID   = 0xffff
-    PRODUCT_ID  = 0xffff
-    BCD         = 0xffff
+    VENDOR_ID   = [0xffff]
+    PRODUCT_ID  = [0xffff]
+    BCD         = [0xffff]
     DEVICE_PLUGBOARD_NAME = 'FOLDER_DEVICE'
 
 
@@ -34,9 +34,9 @@ class FOLDER_DEVICE(USBMS):
     supported_platforms = ['windows', 'osx', 'linux']
     FORMATS     = FOLDER_DEVICE_FOR_CONFIG.FORMATS
 
-    VENDOR_ID   = 0xffff
-    PRODUCT_ID  = 0xffff
-    BCD         = 0xffff
+    VENDOR_ID   = [0xffff]
+    PRODUCT_ID  = [0xffff]
+    BCD         = [0xffff]
     DEVICE_PLUGBOARD_NAME = 'FOLDER_DEVICE'
 
     THUMBNAIL_HEIGHT = 68 # Height for thumbnails on device
