@@ -109,9 +109,11 @@ class OCFZipReader(OCFReader):
             raise EPubException("not a ZIP .epub OCF container")
         self.root = root
         if self.root is None:
-            self.root = os.getcwdu()
-            if hasattr(stream, 'name'):
-                self.root = os.path.abspath(os.path.dirname(stream.name))
+            name = getattr(stream, 'name', False)
+            if name:
+                self.root = os.path.abspath(os.path.dirname(name))
+            else:
+                self.root = os.getcwdu()
         super(OCFZipReader, self).__init__()
 
     def open(self, name, mode='r'):
@@ -164,10 +166,10 @@ def get_cover(opf, opf_path, stream, reader=None):
             return render_html_svg_workaround(cpage, default_log)
 
 def get_metadata(stream, extract_cover=True):
-    """ Return metadata as a :class:`MetaInformation` object """
+    """ Return metadata as a :class:`Metadata` object """
     stream.seek(0)
     reader = OCFZipReader(stream)
-    mi = MetaInformation(reader.opf)
+    mi = reader.opf.to_book_metadata()
     if extract_cover:
         try:
             cdata = get_cover(reader.opf, reader.opf_path, stream, reader=reader)
@@ -176,6 +178,7 @@ def get_metadata(stream, extract_cover=True):
         except:
             import traceback
             traceback.print_exc()
+    mi.timestamp = None
     return mi
 
 def get_quick_metadata(stream):
