@@ -8,12 +8,11 @@ __license__   = 'GPL v3'
 __copyright__ = '2009, John Schember <john@nachtimwald.com>'
 __docformat__ = 'restructuredtext en'
 
-import os
 import struct
 
+from cStringIO import StringIO
+
 from calibre.ebooks.pdb.formatreader import FormatReader
-from calibre.ebooks.txt.processor import convert_basic, opf_writer, \
-    separate_paragraphs_single_line, separate_paragraphs_print_formatted
 
 class HeaderRecord(object):
     '''
@@ -33,9 +32,7 @@ class Reader(FormatReader):
     def __init__(self, header, stream, log, options):
         self.stream = stream
         self.log = log
-        self.encoding = options.input_encoding
-        self.single_line_paras = options.single_line_paras
-        self.print_formatted_paras = options.print_formatted_paras
+        self.options = options
 
         self.sections = []
         for i in range(header.num_sections):
@@ -48,34 +45,29 @@ class Reader(FormatReader):
 
     def decompress_text(self, number):
         if self.header_record.compression == 1:
-            return self.section_data(number).decode('cp1252' if self.encoding is None else self.encoding)
+            return self.section_data(number)
         if self.header_record.compression == 2 or self.header_record.compression == 258:
             from calibre.ebooks.compression.palmdoc import decompress_doc
-            return decompress_doc(self.section_data(number)).decode('cp1252' if self.encoding is None else self.encoding, 'replace')
+            return decompress_doc(self.section_data(number))
         return ''
 
     def extract_content(self, output_dir):
-        txt = ''
+        raw_txt = ''
 
         self.log.info('Decompressing text...')
         for i in range(1, self.header_record.num_records + 1):
             self.log.debug('\tDecompressing text section %i' % i)
-            txt += self.decompress_text(i)
+            raw_txt += self.decompress_text(i)
 
         self.log.info('Converting text to OEB...')
-        if self.single_line_paras:
-            txt = separate_paragraphs_single_line(txt)
-        if self.print_formatted_paras:
-            txt = separate_paragraphs_print_formatted(txt)
-        html = convert_basic(txt)
-        with open(os.path.join(output_dir, 'index.html'), 'wb') as index:
-            index.write(html.encode('utf-8'))
+        stream = StringIO(raw_txt)
 
-        from calibre.ebooks.metadata.meta import get_metadata
-        mi = get_metadata(self.stream, 'pdb')
-        manifest = [('index.html', None)]
-        spine = ['index.html']
-        opf_writer(output_dir, 'metadata.opf', manifest, spine, mi)
+        from calibre.customize.ui import plugin_for_input_format
 
-        return os.path.join(output_dir, 'metadata.opf')
+        txt_plugin = plugin_for_input_format('txt')
+        for option in txt_plugin.options:
+            if not hasattr(self.options, option.option.name):
+                setattr(self.options, option.name, option.recommended_value)
 
+        stream.seek(0)
+        return txt_plugin.convert(stream, self.options, 'txt', self.log, {})
