@@ -72,8 +72,8 @@ class PML_HTMLizer(object):
         'ra': ('<span id="r%s"></span><a href="#%s">', '</a>'),
         'c': ('<div style="text-align: center; margin: auto;">', '</div>'),
         'r': ('<div style="text-align: right;">', '</div>'),
-        't': ('<div style="text-indent: 5%;">', '</div>'),
-        'T': ('<div style="text-indent: %s;">', '</div>'),
+        't': ('<div style="margin-left: 5%;">', '</div>'),
+        'T': ('<div style="margin-left: %s;">', '</div>'),
         'i': ('<span style="font-style: italic;">', '</span>'),
         'u': ('<span style="text-decoration: underline;">', '</span>'),
         'd': ('<span style="text-decoration: line-through;">', '</span>'),
