@@ -9,7 +9,7 @@ from PyQt4.Qt import QVariant, QFileInfo, QObject, SIGNAL, QBuffer, Qt, \
                          QByteArray, QTranslator, QCoreApplication, QThread, \
                          QEvent, QTimer, pyqtSignal, QDate, QDesktopServices, \
                          QFileDialog, QMessageBox, QPixmap, QFileIconProvider, \
-                         QIcon, QApplication, QDialog, QPushButton, QUrl
+                         QIcon, QApplication, QDialog, QPushButton, QUrl, QFont
 
 ORG_NAME = 'KovidsBrain'
 APP_UID  = 'libprs500'
@@ -52,6 +52,9 @@ gprefs.defaults['show_splash_screen'] = True
 gprefs.defaults['toolbar_icon_size'] = 'medium'
 gprefs.defaults['toolbar_text'] = 'auto'
 gprefs.defaults['show_child_bar'] = False
+gprefs.defaults['font'] = None
+gprefs.defaults['tags_browser_partition_method'] = 'first letter'
+gprefs.defaults['tags_browser_collapse_at'] = 100
 
 # }}}
 
@@ -82,7 +85,7 @@ def _config():
     c.add_opt('LRF_ebook_viewer_options', default=None,
               help=_('Options for the LRF ebook viewer'))
     c.add_opt('internally_viewed_formats', default=['LRF', 'EPUB', 'LIT',
-        'MOBI', 'PRC', 'HTML', 'FB2', 'PDB', 'RB'],
+        'MOBI', 'PRC', 'HTML', 'FB2', 'PDB', 'RB', 'SNB'],
               help=_('Formats that are viewed using the internal viewer'))
     c.add_opt('column_map', default=ALL_COLUMNS,
               help=_('Columns to be displayed in the book list'))
@@ -123,6 +126,8 @@ def _config():
             help=_('Download social metadata (tags/rating/etc.)'))
     c.add_opt('overwrite_author_title_metadata', default=True,
             help=_('Overwrite author and title with new metadata'))
+    c.add_opt('auto_download_cover', default=False,
+            help=_('Automatically download the cover, if available'))
     c.add_opt('enforce_cpu_limit', default=True,
             help=_('Limit max simultaneous jobs to number of CPUs'))
     c.add_opt('tag_browser_hidden_categories', default=set(),
@@ -537,6 +542,7 @@ def choose_dir(window, name, title, default_dir='~'):
             parent=window, name=name, mode=QFileDialog.Directory,
             default_dir=default_dir)
     dir = fd.get_files()
+    fd.setParent(None)
     if dir:
         return dir[0]
 
@@ -557,6 +563,7 @@ def choose_files(window, name, title,
     fd = FileDialog(title=title, name=name, filters=filters,
                     parent=window, add_all_files_filter=all_files, mode=mode,
                     )
+    fd.setParent(None)
     if fd.accepted:
         return fd.get_files()
     return None
@@ -567,6 +574,7 @@ def choose_images(window, name, title, select_only_single_file=True):
                     filters=[('Images', ['png', 'gif', 'jpeg', 'jpg', 'svg'])],
                     parent=window, add_all_files_filter=False, mode=mode,
                     )
+    fd.setParent(None)
     if fd.accepted:
         return fd.get_files()
     return None
@@ -611,6 +619,10 @@ class Application(QApplication):
         qt_app = self
         self._file_open_paths = []
         self._file_open_lock = RLock()
+        self.original_font = QFont(QApplication.font())
+        fi = gprefs['font']
+        if fi is not None:
+            QApplication.setFont(QFont(*fi))
 
     def _send_file_open_events(self):
         with self._file_open_lock:
