@@ -16,6 +16,7 @@ from calibre.gui2.dialogs.metadata_bulk import MetadataBulkDialog
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.dialogs.tag_list_editor import TagListEditor
 from calibre.gui2.actions import InterfaceAction
+from calibre.utils.icu import sort_key
 
 class EditMetadataAction(InterfaceAction):
 
@@ -53,6 +54,10 @@ class EditMetadataAction(InterfaceAction):
         mb.addAction(_('Merge into first selected book - keep others'),
                 partial(self.merge_books, safe_merge=True),
                 Qt.AltModifier+Qt.Key_M)
+        mb.addSeparator()
+        mb.addAction(_('Merge only formats into first selected book - delete others'),
+                partial(self.merge_books, merge_only_formats=True),
+                Qt.AltModifier+Qt.ShiftModifier+Qt.Key_M)
         self.merge_menu = mb
         self.action_merge.setMenu(mb)
         md.addSeparator()
@@ -146,13 +151,20 @@ class EditMetadataAction(InterfaceAction):
 
             d = MetadataSingleDialog(self.gui, row_list[current_row], db,
                     prev=prev, next_=next_)
-            if d.exec_() != d.Accepted:
+            d.view_format.connect(lambda
+                    fmt:self.gui.iactions['View'].view_format(row_list[current_row],
+                        fmt))
+            ret = d.exec_()
+            d.break_cycles()
+            if ret != d.Accepted:
                 break
+
             changed.add(d.id)
             if d.row_delta == 0:
                 break
             current_row += d.row_delta
 
+
         if changed:
             self.gui.library_view.model().refresh_ids(list(changed))
             current = self.gui.library_view.currentIndex()
@@ -200,7 +212,7 @@ class EditMetadataAction(InterfaceAction):
             self.gui.library_view.select_rows(ids)
 
     # Merge books {{{
-    def merge_books(self, safe_merge=False):
+    def merge_books(self, safe_merge=False, merge_only_formats=False):
         '''
         Merge selected books in library.
         '''
@@ -214,6 +226,12 @@ class EditMetadataAction(InterfaceAction):
             return error_dialog(self.gui, _('Cannot merge books'),
                         _('At least two books must be selected for merging'),
                         show=True)
+        if len(rows) > 5:
+            if not confirm('<p>'+_('You are about to merge more than 5 books.  '
+                                    'Are you <b>sure</b> you want to proceed?')
+                                +'</p>', 'merge_too_many_books', self.gui):
+                return
+
         dest_id, src_books, src_ids = self.books_to_merge(rows)
         title = self.gui.library_view.model().db.title(dest_id, index_is_id=True)
         if safe_merge:
@@ -228,6 +246,22 @@ class EditMetadataAction(InterfaceAction):
                 return
             self.add_formats(dest_id, src_books)
             self.merge_metadata(dest_id, src_ids)
+        elif merge_only_formats:
+            if not confirm('<p>'+_(
+                'Book formats from the selected books will be merged '
+                'into the <b>first selected book</b> (%s). '
+                'Metadata in the first selected book will not be changed.'
+                'Author, Title, ISBN and all other metadata will <i>not</i> be merged.<br><br>'
+                'After merger the second and subsequently '
+                'selected books, with any metadata they have will be <b>deleted</b>. <br><br>'
+                'All book formats of the first selected book will be kept '
+                'and any duplicate formats in the second and subsequently selected books '
+                'will be permanently <b>deleted</b> from your calibre library.<br><br>  '
+                'Are you <b>sure</b> you want to proceed?')%title
+            +'</p>', 'merge_only_formats', self.gui):
+                return
+            self.add_formats(dest_id, src_books)
+            self.delete_books_after_merge(src_ids)
         else:
             if not confirm('<p>'+_(
                 'Book formats and metadata from the selected books will be merged '
@@ -237,15 +271,10 @@ class EditMetadataAction(InterfaceAction):
                 'subsequently selected books will be <b>deleted</b>. <br><br>'
                 'All book formats of the first selected book will be kept '
                 'and any duplicate formats in the second and subsequently selected books '
-                'will be permanently <b>deleted</b> from your computer.<br><br>  '
+                'will be permanently <b>deleted</b> from your calibre library.<br><br>  '
                 'Are you <b>sure</b> you want to proceed?')%title
             +'</p>', 'merge_books', self.gui):
                 return
-            if len(rows)>5:
-                if not confirm('<p>'+_('You are about to merge more than 5 books.  '
-                                        'Are you <b>sure</b> you want to proceed?')
-                                    +'</p>', 'merge_too_many_books', self.gui):
-                    return
             self.add_formats(dest_id, src_books)
             self.merge_metadata(dest_id, src_ids)
             self.delete_books_after_merge(src_ids)
@@ -363,8 +392,7 @@ class EditMetadataAction(InterfaceAction):
     def edit_device_collections(self, view, oncard=None):
         model = view.model()
         result = model.get_collections_with_ids()
-        compare = (lambda x,y:cmp(x.lower(), y.lower()))
-        d = TagListEditor(self.gui, tag_to_match=None, data=result, compare=compare)
+        d = TagListEditor(self.gui, tag_to_match=None, data=result, key=sort_key)
         d.exec_()
         if d.result() == d.Accepted:
             to_rename = d.to_rename # dict of new text to old ids
