@@ -6,11 +6,11 @@ __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import textwrap
+import textwrap, codecs
 from functools import partial
 
 from PyQt4.Qt import QWidget, QSpinBox, QDoubleSpinBox, QLineEdit, QTextEdit, \
-    QCheckBox, QComboBox, Qt, QIcon, pyqtSignal
+    QCheckBox, QComboBox, Qt, QIcon, pyqtSignal, QLabel
 
 from calibre.customize.conversion import OptionRecommendation
 from calibre.ebooks.conversion.config import load_defaults, \
@@ -81,6 +81,21 @@ class Widget(QWidget):
         self.apply_recommendations(defaults)
         self.setup_help(get_help)
 
+        def process_child(child):
+            for g in child.children():
+                if isinstance(g, QLabel):
+                    buddy = g.buddy()
+                    if buddy is not None and hasattr(buddy, '_help'):
+                        g._help = buddy._help
+                        htext = unicode(buddy.toolTip()).strip()
+                        g.setToolTip(htext)
+                        g.setWhatsThis(htext)
+                        g.__class__.enterEvent = lambda obj, event: self.set_help(getattr(obj, '_help', obj.toolTip()))
+                else:
+                    process_child(g)
+        process_child(self)
+
+
     def restore_defaults(self, get_option):
         defaults = GuiRecommendations()
         defaults.merge_recommendations(get_option, OptionRecommendation.LOW,
@@ -113,6 +128,7 @@ class Widget(QWidget):
     def get_value(self, g):
         from calibre.gui2.convert.xpath_wizard import XPathEdit
         from calibre.gui2.convert.regex_builder import RegexEdit
+        from calibre.gui2.widgets import EncodingComboBox
         ret = self.get_value_handler(g)
         if ret != 'this is a dummy return value, xcswx1avcx4x':
             return ret
@@ -124,6 +140,13 @@ class Widget(QWidget):
             if not ans:
                 ans = None
             return ans
+        elif isinstance(g, EncodingComboBox):
+            ans = unicode(g.currentText()).strip()
+            try:
+                codecs.lookup(ans)
+            except:
+                ans = ''
+            return ans
         elif isinstance(g, QComboBox):
             return unicode(g.currentText())
         elif isinstance(g, QCheckBox):
@@ -168,6 +191,7 @@ class Widget(QWidget):
     def set_value(self, g, val):
         from calibre.gui2.convert.xpath_wizard import XPathEdit
         from calibre.gui2.convert.regex_builder import RegexEdit
+        from calibre.gui2.widgets import EncodingComboBox
         if self.set_value_handler(g, val):
             return
         if isinstance(g, (QSpinBox, QDoubleSpinBox)):
@@ -176,6 +200,11 @@ class Widget(QWidget):
             if not val: val = ''
             getattr(g, 'setPlainText', g.setText)(val)
             getattr(g, 'setCursorPosition', lambda x: x)(0)
+        elif isinstance(g, EncodingComboBox):
+            if val:
+                g.setEditText(val)
+            else:
+                g.setCurrentIndex(0)
         elif isinstance(g, QComboBox) and val:
             idx = g.findText(val, Qt.MatchFixedString)
             if idx < 0:
