@@ -57,6 +57,11 @@ class BooksView(QTableView): # {{{
         elif tweaks['doubleclick_on_library_view'] == 'open_viewer':
             self.setEditTriggers(self.SelectedClicked|self.editTriggers())
             self.doubleClicked.connect(parent.iactions['View'].view_triggered)
+        elif tweaks['doubleclick_on_library_view'] == 'edit_metadata':
+            # Must not enable single-click to edit, or the field will remain
+            # open in edit mode underneath the edit metadata dialog
+            self.doubleClicked.connect(
+                        partial(parent.iactions['Edit Metadata'].edit_metadata, checked=False))
 
         self.drag_allowed = True
         self.setDragEnabled(True)
@@ -105,7 +110,8 @@ class BooksView(QTableView): # {{{
         hv.setCursor(Qt.PointingHandCursor)
         self.selected_ids = []
         self._model.about_to_be_sorted.connect(self.about_to_be_sorted)
-        self._model.sorting_done.connect(self.sorting_done)
+        self._model.sorting_done.connect(self.sorting_done,
+                type=Qt.QueuedConnection)
 
     # Column Header Context Menu {{{
     def column_header_context_handler(self, action=None, column=None):
@@ -122,8 +128,8 @@ class BooksView(QTableView): # {{{
         elif action == 'show':
             h.setSectionHidden(idx, False)
             if h.sectionSize(idx) < 3:
-               sz = h.sectionSizeHint(idx)
-               h.resizeSection(idx, sz)
+                sz = h.sectionSizeHint(idx)
+                h.resizeSection(idx, sz)
         elif action == 'ascending':
             self.sortByColumn(idx, Qt.AscendingOrder)
         elif action == 'descending':
@@ -159,7 +165,7 @@ class BooksView(QTableView): # {{{
                     partial(self.column_header_context_handler,
                         action='descending', column=col))
             if self._model.sorted_on[0] == col:
-                ac = a if self._model.sorted_on[1] == Qt.AscendingOrder else d
+                ac = a if self._model.sorted_on[1] else d
                 ac.setCheckable(True)
                 ac.setChecked(True)
             if col not in ('ondevice', 'rating', 'inlibrary') and \
@@ -227,6 +233,7 @@ class BooksView(QTableView): # {{{
             sm = self.selectionModel()
             for idx in indices:
                 sm.select(idx, sm.Select|sm.Rows)
+            self.scroll_to_row(indices[0].row())
         self.selected_ids = []
     # }}}
 
@@ -275,17 +282,21 @@ class BooksView(QTableView): # {{{
     def cleanup_sort_history(self, sort_history):
         history = []
         for col, order in sort_history:
+            if not isinstance(order, bool):
+                continue
             if col == 'date':
                 col = 'timestamp'
-            if col in self.column_map and (not history or history[0][0] != col):
-                history.append([col, order])
+            if col in self.column_map:
+                if (not history or history[-1][0] != col):
+                    history.append([col, order])
         return history
 
     def apply_sort_history(self, saved_history):
         if not saved_history:
             return
         for col, order in reversed(self.cleanup_sort_history(saved_history)[:3]):
-            self.sortByColumn(self.column_map.index(col), order)
+            self.sortByColumn(self.column_map.index(col),
+                              Qt.AscendingOrder if order else Qt.DescendingOrder)
 
     def apply_state(self, state):
         h = self.column_header
@@ -375,7 +386,12 @@ class BooksView(QTableView): # {{{
             old_state = self.get_default_state()
 
         if tweaks['sort_columns_at_startup'] is not None:
-            old_state['sort_history'] = tweaks['sort_columns_at_startup']
+            sh = []
+            for c,d in tweaks['sort_columns_at_startup']:
+                if not isinstance(d, bool):
+                    d = True if d == 0 else False
+                sh.append((c, d))
+            old_state['sort_history'] = sh
 
         self.apply_state(old_state)
 
@@ -596,7 +612,7 @@ class BooksView(QTableView): # {{{
         if row > -1:
             h = self.horizontalHeader()
             for i in range(h.count()):
-                if not h.isSectionHidden(i):
+                if not h.isSectionHidden(i) and h.sectionViewportPosition(i) >= 0:
                     self.scrollTo(self.model().index(row, i))
                     break
 
