@@ -112,13 +112,19 @@ _extra_lang_codes = {
         'en_CN' : _('English (China)'),
         'es_PY' : _('Spanish (Paraguay)'),
         'de_AT' : _('German (AT)'),
+        'fr_BE' : _('French (BE)'),
         'nl'    : _('Dutch (NL)'),
         'nl_BE' : _('Dutch (BE)'),
         'und'   : _('Unknown')
         }
 
+_lcase_map = {}
+for k in _extra_lang_codes:
+    _lcase_map[k.lower()] = k
+
 def get_language(lang):
     global _iso639
+    lang = _lcase_map.get(lang, lang)
     if lang in _extra_lang_codes:
         return _extra_lang_codes[lang]
     ip = P('localization/iso639.pickle')
