@@ -1087,7 +1087,9 @@ class ZipFile:
                     with open(targetpath, 'wb') as target:
                         shutil.copyfileobj(source, target)
                 except:
-                    targetpath = sanitize_file_name(targetpath)
+                    components = list(os.path.split(targetpath))
+                    components[-1] = sanitize_file_name(components[-1])
+                    targetpath = os.sep.join(components)
                     with open(targetpath, 'wb') as target:
                         shutil.copyfileobj(source, target)
         self.extract_mapping[member.filename] = targetpath
@@ -1227,7 +1229,7 @@ class ZipFile:
         self.fp.flush()
         if zinfo.flag_bits & 0x08:
             # Write CRC and file sizes after the file data
-            self.fp.write(struct.pack("<lLL", zinfo.CRC, zinfo.compress_size,
+            self.fp.write(struct.pack("<LLL", zinfo.CRC, zinfo.compress_size,
                   zinfo.file_size))
         self.filelist.append(zinfo)
         self.NameToInfo[zinfo.filename] = zinfo
@@ -1256,6 +1258,12 @@ class ZipFile:
         """Call the "close()" method in case the user forgot."""
         self.close()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, typ, value, traceback):
+        self.close()
+
     def close(self):
         """Close the file, and for mode "w" and "a" write the ending
         records."""
